@@ -48,6 +48,7 @@ from nwg_panel.modules.scratchpad import Scratchpad
 from nwg_panel.modules.dwl_tags import DwlTags
 from nwg_panel.modules.swaync import SwayNC
 from nwg_panel.modules.sway_mode import SwayMode
+from nwg_panel.modules.hyprland_submap import HyprlandSubmap
 from nwg_panel.modules.keyboard_layout import KeyboardLayout
 from nwg_panel.modules.random_wallpaper import RandomWallpaper
 
@@ -89,6 +90,10 @@ if his:
     from nwg_panel.modules.hyprland_taskbar import HyprlandTaskbar
     from nwg_panel.modules.hyprland_workspaces import HyprlandWorkspaces
 
+niri_sock = os.getenv('NIRI_SOCKET')
+if niri_sock:
+    from nwg_panel.modules.niri_taskbar import NiriTaskbar
+
 common_settings = {}
 restart_cmd = ""
 sig_dwl = 0
@@ -163,7 +168,6 @@ def hypr_watcher():
 
     client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
     client.connect(f"{hypr_dir}/{his}/.socket2.sock")
-    just_refreshed = False
 
     while True:
         datagram = client.recv(2048)
@@ -176,6 +180,10 @@ def hypr_watcher():
         # print(f"events: {event_names}")
 
         for event_name in event_names:
+            if common_settings["restart-on-display"] and (event_name in ["monitoradded", "monitorremoved"]):
+                print("Received event '{}'; restart in {} ms.".format(event_name, common_settings["restart-delay"]))
+                GLib.timeout_add(common_settings["restart-delay"], restart, priority=GLib.PRIORITY_HIGH)
+
             if event_name in ["activespecial",
                               "activewindow",
                               "activewindowv2",
@@ -190,10 +198,6 @@ def hypr_watcher():
                               "windowtitle",
                               "workspace"]:
 
-                if "activewindow" in event_name and just_refreshed:
-                    just_refreshed = False
-                    break
-
                 # print(f">>> refreshing on {event_name}")
                 monitors, workspaces, clients, activewindow, activeworkspace = h_modules_get_all()
                 for item in common.h_taskbars_list:
@@ -201,11 +205,44 @@ def hypr_watcher():
 
                 for item in common.h_workspaces_list:
                     GLib.timeout_add(0, item.refresh, monitors, workspaces, clients, activewindow, activeworkspace)
-
-                if event_name in ["createworkspace", "destroyworkspace", "focusedmon", "workspace"]:
-                    just_refreshed = True
                 break
 
+            elif event_name == "submap":
+                for item in common.h_submaps_list:
+                    GLib.timeout_add(0, item.refresh)
+
+
+def niri_watcher():
+    import socket
+
+    client = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+    client.connect(niri_sock)
+    client.sendall(b"\"EventStream\"\n")
+
+    buffer = ""
+    while True:
+        chunk = client.recv(1024).decode('utf-8', errors='replace')
+        if not chunk:
+            break
+        buffer += chunk
+        while "\n" in buffer:
+            line, buffer = buffer.split("\n", 1)
+            line = line.strip()
+            if not line:
+                continue
+            try:
+                message = json.loads(line)
+                # event_name = next(iter(message))
+                # event_data = message[event_name]
+                # print(f"[{event_name}]")
+
+                outputs, workspaces, windows, focused_window = niri_get_all()
+                # print(f"{outputs}\n{workspaces}\n{windows}\n{focused_window}")
+                for item in common.niri_taskbars_list:
+                    GLib.timeout_add(0, item.refresh, outputs, workspaces, windows, focused_window)
+
+            except json.JSONDecodeError as e:
+                print("Failed to decode JSON:", e)
 
 def on_i3ipc_event(i3conn, event):
     if common_settings["restart-on-display"]:
@@ -296,11 +333,6 @@ def instantiate_content(panel, container, content_list, icons_path=""):
 
         if item == "hyprland-taskbar":
             if "hyprland-taskbar" in panel:
-                # if panel["layer"] in ["bottom", "background"]:
-                #     eprint(
-                #         "Panel '{}': On Hyprland, panels must be placed on 'top' or 'overlay' layer, but '{}' found in "
-                #         "settings. Changing to 'top'.".format(panel["name"], panel["layer"]))
-                #     panel["layer"] = "top"  # or context menu will remain invisible
                 if his:
                     check_key(panel["hyprland-taskbar"], "all-outputs", False)
                     if panel["hyprland-taskbar"]["all-outputs"] or "output" not in panel:
@@ -316,10 +348,30 @@ def instantiate_content(panel, container, content_list, icons_path=""):
                 else:
                     eprint("'hyprland-taskbar' ignored (HIS unknown).")
 
+        if item == "niri-taskbar":
+            if niri_sock:
+                outputs, workspaces, windows, focused_window = {}, {}, {}, {}
+                if "niri-taskbar" in panel:
+                    if niri_sock:
+                        check_key(panel, "niri-taskbar", {})
+                        check_key(panel["niri-taskbar"], "all-outputs", False)
+                        if panel["niri-taskbar"]["all-outputs"] or "output" not in panel:
+                            taskbar = NiriTaskbar(panel["niri-taskbar"], panel["position"], outputs, workspaces,
+                                                      windows, focused_window, icons_path=icons_path)
+                        else:
+                            taskbar = NiriTaskbar(panel["niri-taskbar"], panel["position"], outputs, workspaces,
+                                                      windows, focused_window, display_name="{}".format(panel["output"]),
+                                                      icons_path=icons_path)
+
+                        common.niri_taskbars_list.append(taskbar)
+                        container.pack_start(taskbar, False, False, panel["items-padding"])
+                    else:
+                        eprint("'niri-taskbar' ignored (NIRI_SOCKET unknown).")
+
         if item == "hyprland-workspaces":
             if his:
                 if "hyprland-workspaces" in panel:
-                    workspaces = HyprlandWorkspaces(panel["hyprland-workspaces"], monitors, workspaces, clients,
+                    workspaces = HyprlandWorkspaces(panel["hyprland-workspaces"], panel["output"], monitors, workspaces, clients,
                                                     activewindow, activeworkspace, icons_path=icons_path)
                     container.pack_start(workspaces, False, False, panel["items-padding"])
                     common.h_workspaces_list.append(workspaces)
@@ -328,14 +380,25 @@ def instantiate_content(panel, container, content_list, icons_path=""):
             else:
                 eprint("'hyprland-workspaces' ignored")
 
+        if item == "hyprland-submap":
+            if his:
+                if item in panel:
+                    h_submap = HyprlandSubmap(panel[item], icons_path=icons_path)
+                else:
+                    h_submap = HyprlandSubmap({}, icons_path=icons_path)
+                container.pack_start(h_submap, False, False, panel["items-padding"])
+                common.h_submaps_list.append(h_submap)
+            else:
+                eprint("HyprlandSubmap module only works on Hyprland, ignoring")
+
         if item == "keyboard-layout":
-            if his or sway:
+            if his or sway or niri_sock:
                 if "keyboard-layout" not in panel:
                     panel["keyboard-layout"] = {}
                 kb_layout = KeyboardLayout(panel["keyboard-layout"], icons_path)
                 container.pack_start(kb_layout, False, False, panel["items-padding"])
             else:
-                eprint("KeyboardLayout module does not yet support sway")
+                eprint("KeyboardLayout module does not yet support your compositor")
 
         if "button-" in item:
             if item in panel:
@@ -536,9 +599,6 @@ def main():
 
     save_string("-c {} -s {}".format(args.config, args.style), os.path.join(local_dir(), "args"))
 
-    common.app_dirs = get_app_dirs()
-    # common.name2icon_dict = map_odd_desktop_files()
-
     config_file = os.path.join(common.config_dir, args.config)
 
     copy_files(os.path.join(dir_name, "icons_light"), os.path.join(common.config_dir, "icons_light"))
@@ -551,7 +611,9 @@ def main():
 
     # tree = common.i3.get_tree() if sway else None
     common.outputs, common.mon_desc2output_name = list_outputs(sway=sway)
-    print(f"Outputs: {common.outputs}")
+    print("Outputs:")
+    for key in common.outputs:
+        print(key, common.outputs[key])
     print(f"Descriptions: {common.mon_desc2output_name}")
 
     panels = load_json(config_file)
@@ -576,6 +638,8 @@ def main():
     for panel in panels:
         check_key(panel, "output", "")
         check_key(panel, "monitor", "")
+        check_key(panel, "run-if-output-exist", [])
+        check_key(panel, "run-if-output-absent", [])
         if panel["monitor"]:
             try:
                 panel["output"] = common.mon_desc2output_name[panel["monitor"]]
@@ -591,9 +655,17 @@ def main():
                 clones.append(clone)
 
             to_append = to_append + clones
+        else:
+            for output in panel["run-if-output-exist"]:
+                if output not in common.outputs:
+                    to_remove.append(panel)
+            for output in panel["run-if-output-absent"]:
+                if output in common.outputs:
+                    to_remove.append(panel)
 
     for item in to_remove:
-        panels.remove(item)
+        if item in panels:
+            panels.remove(item)
 
     panels = panels + to_append
 
@@ -605,7 +677,7 @@ def main():
             pass
 
         if panel["output"] and not monitor:
-            print("Couldn't assign a Gdk.Monitor to output '{}'".format(panel["output"]))
+            eprint("Couldn't assign a Gdk.Monitor to output '{}'".format(panel["output"]))
             continue
 
         check_key(panel, "icons", "")
@@ -885,6 +957,11 @@ def main():
             thread.daemon = True
             thread.start()
 
+    if niri_sock:
+        thread = threading.Thread(target=niri_watcher, daemon=True)
+        thread.daemon = True
+        thread.start()
+
     if tray_available and len(common.tray_list) > 0:
         sni_system_tray.init_tray(common.tray_list)
 
